package ru.dbotthepony.mc.otm.worldgen.placement

import com.google.common.collect.ImmutableList
import com.mojang.serialization.Codec
import com.mojang.serialization.MapCodec
import net.minecraft.core.BlockPos
import net.minecraft.util.RandomSource
import net.minecraft.world.level.levelgen.placement.PlacementContext
import net.minecraft.world.level.levelgen.placement.PlacementModifier
import net.minecraft.world.level.levelgen.placement.PlacementModifierType
import ru.dbotthepony.mc.otm.registry.data.MPlacementModifiers
import java.util.stream.Stream

/**
 * Or "shard" placement, if you will.
 *
 * Basically, allows multiple [PlacementModifier]s to split/branch off from provided point.
 */
class SplitPlacement(
	val children: List<PlacementModifier>
) : PlacementModifier() {
	constructor(vararg children: PlacementModifier) : this(ImmutableList.copyOf(children))

	override fun getPositions(context: PlacementContext, random: RandomSource, pos: BlockPos): Stream<BlockPos> {
		return children.stream().flatMap { it.getPositions(context, random, pos) }
	}

	override fun type(): PlacementModifierType<*> {
		return MPlacementModifiers.SPLIT
	}

	companion object {
		val CODEC: MapCodec<SplitPlacement> = Codec.list(PlacementModifier.CODEC).xmap(::SplitPlacement, SplitPlacement::children).fieldOf("children")
	}
}
